const { check, body, validationResult } = require("express-validator");
const express = require("express");
const peopleRouter = express.Router();
const { createPerson } = require("../service/peopleService");
/*
Function to check that the POST request body in 
create person contains the correct data types 
*/
const validateCreatePerson = [
  // Validate personInfo
  check("personInfo.firstName")
    .notEmpty()
    .withMessage("First name is required")
    .isString()
    .withMessage("First name must be a string"),

  check("personInfo.lastName")
    .notEmpty()
    .withMessage("Last name is required")
    .isString()
    .withMessage("Last name must be a string"),

  check("personInfo.classYear")
    .notEmpty()
    .withMessage("Class year is required")
    .isInt()
    .withMessage("Class year must be an integer"),

  // check if greater than 0
  check("personInfo.gradYear")
    .optional()
    .isInt()
    .withMessage("Grad year must be an integer if provided"),

  check("personInfo.gradSemester")
    .optional()
    .isString()
    .withMessage("Grad semester must be an String if provided"),

  check("personInfo.gender")
    .isIn(["male", "female", "other"])
    .withMessage("Invalid gender"),

  check("personInfo.maidenName")
    .optional()
    .isString()
    .withMessage("Maiden name should be a string"),

  // Validate degrees array
  body("degrees").isArray().withMessage("Degrees should be an array"),

  body("degrees.*.degreeType").isString().withMessage("Invalid degree type"),

  body("degrees.*.degreeYear")
    .notEmpty()
    .withMessage("Degree year is required")
    .isInt()
    .withMessage("Degree year must be an integer"),

  body("degrees.*.degreeDescription")
    .optional()
    .isLength({ max: 255 })
    .withMessage("Degree description cannot exceed 255 characters"),

  // Validate addresses array
  body("addresses").isArray().withMessage("Addresses should be an array"),

  body("addresses.*.address1")
    .notEmpty()
    .withMessage("Address line 1 is required"),

  body("addresses.*.city").notEmpty().withMessage("City is required"),

  body("addresses.*.state").notEmpty().withMessage("State is required"),

  body("addresses.*.country").notEmpty().withMessage("Country is required"),

  body("addresses.*.zipCode")
    .optional()
    .isInt()
    .withMessage("Zip code must be an integer if provided"),

  // Validate contacts array
  body("contacts").isArray().withMessage("Contacts should be an array"),

  body("contacts.*.contactNumber")
    .notEmpty()
    .withMessage("Contact number is required")
    .isString()
    .withMessage("Contact number must be a string"),

  body("contacts.*.contactType")
    .isIn(["phone", "email"])
    .withMessage("Invalid contact type"),

  body("contacts.*.preferredContact")
    .isBoolean()
    .withMessage("Preferred contact must be a boolean"),

  // Validate involvements
  body("involvements")
    .optional()
    .isArray()
    .withMessage("Involvements should be an array"),

  body("involvements.*")
    .optional()
    .isString()
    .withMessage("Each involvement must be a string"),
];

peopleRouter.post("/create", validateCreatePerson, async (req, res) => {
  // ensure the request body has the correct data types
  const errors = validationResult(req);

  // error found: return early
  if (!errors.isEmpty()) {
    return res.status(400).json({ errors: errors.array() });
  }

  try {
    const person = await createPerson(req.body);
    console.log("In people routes after add: ", person);

    return res
      .status(201)
      .json({ message: "person added successfully", personId: person });
  } catch (error) {
    console.log("error");
    return res.status(500).json({ message: error });
  }
});

module.exports = peopleRouter;
