// Mock the database's pool and query function
jest.mock("../../config/database/database.config", () => {
  const queryMock = jest.fn(); // Mock query function
  return { pool: { query: queryMock } }; // Return the mocked pool directly
});

const { pool } = require("../../config/database/database.config.js");
const { addDegree } = require("../../repository/degreeRepository.js");

/*
Test methods for degreeRepository
*/
describe("Testing degreeRepository.js", () => {
  const mockDegree = {
    degreeType: "BS",
    degreeDepartment: "Computer Science",
    degreeCollege: "Engineering",
    degreeYear: "2024",
    degreeDescription: "Bachelor of Science in Computer Science",
  };

  const mockPeopleId = 1;

  /*
  Tests for the addDegree method
  */
  describe("addDegree tests", () => {
    it("Insert a degree with no errors", async () => {
      const mockDegreeTypeId = 1;
      const mockResult = { affectedRows: 1 };

      // Mock the helper function _lookupDegreeTypeId to return 1
      const lookupDegreeTypeIdSpy = jest
        .spyOn(
          require("../../repository/degreeRepository"),
          "_lookupDegreeTypeId"
        )
        .mockResolvedValueOnce(mockDegreeTypeId);

      // Mock the database query for inserting the degree
      pool.query.mockResolvedValueOnce([mockResult]);

      // Call addDegree
      const result = await addDegree(mockDegree, mockPeopleId);

      // Verify the SQL query for inserting the degree
      expect(pool.query).toHaveBeenCalledWith(expect.any(String), [
        mockPeopleId,
        null, // null for now
        "Computer Science",
        "Engineering",
        "2024",
        "Bachelor of Science in Computer Science",
      ]);

      // Expect the function to return the result from the database
      expect(result).toBe(mockResult);

      // Clean up spy
      lookupDegreeTypeIdSpy.mockRestore();
    });

    it("error thrown during degree insertion", async () => {
      const mockError = new Error("Database query failed");

      // Mock the helper function _lookupDegreeTypeId to return 1
      const lookupDegreeTypeIdSpy = jest
        .spyOn(
          require("../../repository/degreeRepository"),
          "_lookupDegreeTypeId"
        )
        .mockResolvedValueOnce(1);

      // Mock the database query to throw an error during degree insertion
      pool.query.mockRejectedValueOnce(mockError);

      // Call addDegree and expect it to handle the error
      const result = await addDegree(mockDegree, mockPeopleId);

      // Expect the function to return null on error
      expect(result).toBeNull();

      // Clean up the spy
      lookupDegreeTypeIdSpy.mockRestore();
    });

    it("handle errors during degree type lookup", async () => {
      const mockError = new Error("Degree type lookup failed");

      // Mock the helper function _lookupDegreeTypeId to throw an error
      const lookupDegreeTypeIdSpy = jest
        .spyOn(
          require("../../repository/degreeRepository"),
          "_lookupDegreeTypeId"
        )
        .mockRejectedValueOnce(mockError);

      // Call addDegree and expect it to handle the error
      const result = await addDegree(mockDegree, mockPeopleId);

      // Expect the function to return null on error
      expect(result).toBeNull();

      // Clean up the spy
      lookupDegreeTypeIdSpy.mockRestore();
    });
  });
});
